package geniusweb.profileconnection;

import java.io.IOException;
import java.net.URI;
import java.util.logging.Level;

import javax.websocket.ClientEndpoint;
import javax.websocket.DeploymentException;
import javax.websocket.OnError;
import javax.websocket.OnMessage;
import javax.websocket.Session;
import javax.websocket.WebSocketContainer;

import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.profile.Profile;
import tudelft.utilities.listener.DefaultListenable;
import tudelft.utilities.logging.Reporter;

/**
 * Container for the latest available profile on the profiles server
 */
@ClientEndpoint
public class WebsocketProfileConnector extends DefaultListenable<Profile>
		implements ProfileInterface {
	private final static ObjectMapper jackson = new ObjectMapper();
	private static final int TIMEOUT_MS = 2000;
	private final Reporter logger;
	private Profile profile = null;

	/**
	 * 
	 * @param uri         the URI that will provide the {@link Profile}
	 * @param reporter    the {@link Reporter} for logging issues
	 * @param wscontainer the {@link WebSocketContainer} that can provide new
	 *                    websockets. Typically ContainerProvider
	 *                    .getWebSocketContainer()
	 * @throws DeploymentException
	 * @throws IOException
	 */
	public WebsocketProfileConnector(URI uri, Reporter reporter,
			WebSocketContainer wscontainer)
			throws DeploymentException, IOException {
		if (uri == null || reporter == null || wscontainer == null) {
			throw new IllegalArgumentException(
					"uri, reporter and wsconnector must be not null");
		}
		this.logger = reporter;
		wscontainer.connectToServer(this, uri);
	}

	public void onOpen(Session session) {
		logger.log(Level.FINE, "Connected: " + session.getBasicRemote());
	}

	@OnMessage
	public void processMessage(String message) throws IOException {
		// this will be called every time the profile changes.
		logger.log(Level.FINE, "Received profile: " + message);
		profile = jackson.readValue(message, Profile.class);
		notifyListeners(profile);
	}

	@OnError
	public void processError(Throwable t) {
		logger.log(Level.SEVERE,
				"Something went wrong while processing downloaded profile", t);
	}

	@Override
	public Profile getProfile() throws IOException {
		int remaining_wait = TIMEOUT_MS;
		try {
			while (profile == null && remaining_wait > 0) {
				remaining_wait -= 100;
				Thread.sleep(100);
			}
		} catch (InterruptedException e) {
			logger.log(Level.SEVERE, "interrupted while waiting for profile",
					e);
		}
		if (profile == null) {
			throw new IOException("Failed to fetch profile");
		}
		return profile;
	}

}